#! /usr/local/perl
#
#   Load PicoWeb EEPROM using network
#
sub usage {
    print STDERR "usage: $ego IPaddr EEPROM_file [blk_size]\n" ;
    print STDERR "       $ego IPaddr -url URL\n" ;
    print STDERR "       $ego IPaddr -port port loc byte1 ... byteN\n" ;
    print STDERR "\@IPaddr indicates name of file with IP address" ;
    print STDERR "-port loads SEEPROM bytes (loc/byteN in hex)\n" ;
    exit 1 ;
}

$debug = 0;
$ego = $0 ;
$ego =~ s/^.*[\/\\]// ;
&usage if ($ARGV[0] eq '') ;
$tmp = 'img.tmp';
$tmp2 = 'img2.tmp';
$is_win = 1; $is_win = 0 if ($ENV{'LINUX'} ne '') ;

$connect_timeout = 3 ;
$read_timeout = 5 ;

$dir = $0 ;
if ($dir =~ /[\/\\]/) {
    $dir =~ s/([\\\/])[^\\\/]+$/$1/;  # use httpget in our directory
} else {
    $dir = './'
}
if ($is_win) {
    $dir =~ s/\//\\/g ;
    $httpget = $dir."httpget -C $connect_timeout -t $read_timeout";
} else {
    $httpget = "httpget -C $connect_timeout -t $read_timeout";
}

$status = 0 ;
$url = '' ;
$port = 0 ;
select((select(STDOUT), $| = 1)[0]);    # unbuffer output
select((select(STDERR), $| = 1)[0]);    # unbuffer output

if ($ARGV[0] eq '-d') {
    $debug = 1;
    shift ;
}

exit(1) if (($ip = &getip($ARGV[0])) eq '');
shift ;

if ($ARGV[0] eq '-url') {
    shift ;
    $url = $ARGV[0] ;
    shift ;
}
if ($ARGV[0] eq '-port') {
    shift ;
    $port = $ARGV[0] ;
    shift ;
}

if ($port > 0) {
    &do_open ;
    $addr = hex($ARGV[0]) ;
    printf(STDERR "loc=%04x:", $addr) if ($debug);
    syswrite(I,pack("S",$addr),2) ;
    for ($i=1 ; $i<=$#ARGV ; $i++)  {
        syswrite(I,pack("C",hex($ARGV[$i])),1) ;
        print STDERR " $ARGV[$i]" if ($debug);
    }
    close(I) ;
    print STDERR "\n" if ($debug) ;

    unlink($tmp2) ;
    $cmd = "$httpget -rP $tmp -o $tmp2 $ip:$port" ;
    print STDERR "$cmd\n" if ($debug);
    $out = `$cmd` ;
    print $out if ($debug) ;
    &check_output;
    exit $status ;
}

if ($url) {
    &usage if ($ARGV[0] ne '') ;
    $cmd = "$httpget $ip$url" ;
    print STDERR "$cmd\n" if ($debug);
    system($cmd) ;
    exit(0) ;
}

&usage if ($ARGV[1] ne '') ;
&do_open ;
open(F,"<$ARGV[0]") || die "Can't open $ARGV[0]" ;
binmode(F) ;
$addr = 0 ;
if ($ARGV[0] =~ /\.el/i)  {
    sysread(F,$buf,2) ;
    $addr = unpack("S",$buf) ;  # starting address
}
$blocksize = $ARGV[1] ;
$blocksize = 192 if ($blocksize == 0) ; # 256-4-6-6-2-20-20-2(length)
#
# blocksize can still be specified, but it better be a multiple of
# the page size in serial eeprom (64 bytes) - otherwise page wrap will
# occur, and the results will not be favorable!
#
while (sysread(F,$buf,$blocksize) > 0)  {
     ###if (length($buf) != $blocksize)  {  # truncate
        ###close(I) ;
        ###&do_open ;
    ###}
    ###if ($is_win) {
        ###seek(I,0,0) ;
    ###} else {
        ###eval 'sysseek(I,0,0);' ;
        ###seek(I,0,0) if ($@ ne '');
    ###}
    syswrite(I,pack("S",$addr),2) ;
    syswrite(I,$buf,length($buf)) ;
    $addr += length($buf) ;
}
close(I) ;

$len = $blocksize + 2 ;
unlink($tmp2) ;
$cmd = "$httpget -rP $tmp -N $len -o $tmp2 $ip:611" ;
printf(STDERR "%04x: $cmd -> ", $addr) if ($debug);
##printf "%04X ",$addr ;
##system($cmd) ;
$s = `$cmd` ;
print STDERR "$s\n" if ($debug);
&check_output ;

unlink($tmp) if ($debug == 0) ;
exit $status ;

sub check_output {
    if ((-z $tmp2) || (!open(I,"<$tmp2"))) {
         print STDERR "Failed to connect to PicoWeb at $ip!\n" ;
         $status = 1 ;
    } else {
        while (<I>) {
            $s = $_ ;
            $s =~ s/[\r\n]+$//;
            if (!($s =~ /^\s+[0-9a-fA-F]+$/)) {
                print "$s" ;
                $status = 1 ;
            }
        }
        close(I) ;
    }
    unlink($tmp2) if ($debug == 0) ;
}

sub do_open  {
    if (!open(I,">$tmp")) {
         print STDERR "Can't create $tmp!\n" ;
         exit 1;
    }
    binmode(I) ;
    select(I) ;
    $| = 1 ;
    select(STDOUT) ;
    $| = 1 ;
}
#
#  Resolve IP address
#  usage: exit(1) if (($ip = &getip($ip)) eq '');
#         where $s is IP address, '-', or @file
#
sub getip {
    local($f) = @_;

    $ip = '';
    if ($f =~ /^[@]/ || $f eq '-') {
        if ($f ne '-') {
            $f =~ s/^[@]//;
            if (!open(IP, "<$f")) {
                print STDERR "Can't open IP address file \"$f\"!\n";
                return '';
            }
        }
        while(1) {
            if ($f ne '-') { $s = <IP>; } else { $s = <>; }
            ##print "$s";
            last if ($s eq '');
            if ($s =~ /^\s*(\d+[.]\d+[.]\d+[.]\d+)[\r\n\s]*$/) {
                $ip = $1;
            }
        }
    } else {
        $ip = $f;
    }
    if ($ip =~ /^\d+[.]\d+[.]\d+[.]\d+$/) {
        return $ip;
    } else {
        print STDERR "IP address ($ip) not of the form x.y.z.w!\n";
        return '';
    }
}
