;
; Copyright (C) 1998
;
;	Stephen D. Freyder, David R. Helland, Bruce D. Lightner.
;
#if defined(DOCUMENTATION)
      FIELD   BYTES   DESCRIPTION
      -----   -----   -----------

         op      1       packet op code / message type.
                         1 = BOOTREQUEST, 2 = BOOTREPLY

         htype   1       hardware address type,
                         see ARP section in "Assigned Numbers" RFC.
                         '1' = 10mb ethernet

         hlen    1       hardware address length
                         (eg '6' for 10mb ethernet).

         hops    1       client sets to zero,
                         optionally used by gateways
                         in cross-gateway booting.

         xid     4       transaction ID, a random number,
                         used to match this boot request with the
                         responses it generates.

         secs    2       filled in by client, seconds elapsed since
                         client started trying to boot.


Croft & Gilmore                                                 [Page 3]



RFC 951                                                   September 1985
Bootstrap Protocol


         --      2       unused

         ciaddr  4       client IP address;
                         filled in by client in bootrequest if known.

         yiaddr  4       'your' (client) IP address;
                         filled by server if client doesn't
                         know its own address (ciaddr was 0).

         siaddr  4       server IP address;
                         returned in bootreply by server.

         giaddr  4       gateway IP address,
                         used in optional cross-gateway booting.

         chaddr  16      client hardware address,
                         filled in by client.

         sname   64      optional server host name,
                         null terminated string.

         file    128     boot file name, null terminated string;
                         'generic' name or null in bootrequest,
                         fully qualified directory-path
                         name in bootreply.

         vend    64      optional vendor-specific area,
                         e.g. could be hardware type/serial on request,
                         or 'capability' / remote file system handle
                         on reply.  This info may be set aside for use
                         by a third phase bootstrap or kernel.
#endif

#define BOOTP_OP        UDP_DATA+0      /* 1 - optype */
#define BOOTP_HTYPE     UDP_DATA+1      /* 1 - hw type */
#define BOOTP_HLEN      UDP_DATA+2      /* 1 - hw addr len */
#define BOOTP_HOPS      UDP_DATA+3      /* 1 - max hops */
#define BOOTP_XID       UDP_DATA+4      /* 4 - transaction id (random) */
#define BOOTP_SECS      UDP_DATA+8      /* 2 - seconds since client boot */
#define BOOTP_UNUSED    UDP_DATA+10     /* 2 - unused (padding?) */
#define BOOTP_CIADDR    UDP_DATA+12     /* 4 - client IP addr (req) or 0.0.0.0 */
#define BOOTP_YIADDR    UDP_DATA+16     /* 4 - your ip addr (reply) */
#define BOOTP_SIADDR    UDP_DATA+20     /* 4 - server ip address (reply) */
#define BOOTP_GIADDR    UDP_DATA+24     /* 4 - gateway ip address */
#define BOOTP_CHADDR    UDP_DATA+28     /* 16- client hw addr (req) */
#define BOOTP_SNAME     UDP_DATA+44     /* 64- asciz hostname (req) */
#define BOOTP_FILE      UDP_DATA+108    /*128- asciz boot file name */
#define BOOTP_VEND      UDP_DATA+236    /* 64- vendor-specific area */
#define BOOTP_LEN       300             /* total length */

#define BOOTP_REQUEST_PORT 67
#define BOOTP_REPLY_PORT   68
