;-------------------------------------------------------------------------
; Copyright (c) 1999 Steven Freyder, David Helland, Bruce D. Lightner and
; Lightner Engineering.  Use subject to licensing restrictions described
; the file LICENSE.txt.  If you do not agree with the licensing terms then
; you must return and/or destroy this software immediately!
;-------------------------------------------------------------------------

.dseg

cmdop1: .byte   2                       ; command optional arguement #1 (word)
cmdop2: .byte   2                       ; command optional arguement #2 (word)
cmd_ok: .byte   1                       ; used by command processor
.cseg

;+
; **-command_check-check for command from serial port.
;-

do_prompt:
	outstr "\r\n# "
no_command:
    ret

import_cli_exec:
cli_exec:

	clr r20                 ; default to bad command
	sts cmd_ok,r20

	rcall pcode
#include "cmd.inc"
    asm

	lds r20,cmd_ok          ; command OK?
	tst r20
	brne cli_exec_ok
	putchar '?'

cli_exec_ok:
	ret

command_check:

;
; if no command, keep looping.
;
	lds     w,ready
	tst     w
	breq    no_command          ; none pending
;
; there is a command in the input buffer, nul-terminate it.
;
	ldsbw z,nch
	addwi z,inbuf
    clr     r17
	st      z,r17               ; nul-terminate
    sts ready,r17
    sts nch,r17                     ; mark buffer empty
;
; process command.
;
	crlf
    rcall cli_exec
    rjmp do_prompt
;+
; **-dumpsram-dump sram in hex.
;-

dump_sram:
	movwi z,readb_y
dumpx:
	ldsw    y,cmdop1
	ldsw    r21,cmdop2          ; get length
    lds     ch,putc_b
    tst     ch
    breq    dumpsr1             ; no limit if going to serial port
    cmpwi    r21,1024
    brlo    dumpsr1
    breq    dumpsr1
    movwi    r21,1024
dumpsr1:
    wdr
	mov     ch,yl
	andi    ch,0x0f
	brne    dumpsr2
	crlf
	rcall   hexy
dumpsr2:
	space
	pushw z
	icall
	popw z
	hexb
        decw r21
        brcs dumpsr_done
        brne dumpsr1
dumpsr_done:
	crlf

	ret

readb_y:
	ld i0,y+
	ret
pcode_debug_toggle: lds yl,cmdop1
    sts pcode_debug_flag,yl
    ret
;+
; **-dump_eeprom-dump eeprom.
;-

dump_eeprom:
	movwi z,rdeeb
	rjmp dumpx
;+
; **-dump_seeprom-dump seeprom.
;-

dump_seeprom:
	movwi z,rdseeb
	rjmp dumpx
;+
; **-do_pcode_call-do a pcode call (pcall)
;
; inputs:
;   cmdop1  = word containing address to call (no checking!!!)
;
; outputs:
;   who knows!!
;-

do_pcode_call:
    pushw p
    rcall pcode
    pcall [cmdop1]
    asm
#if defined(PCODE_DEBUG)
    putchar '>'
    popw y
    rcall hexy
    pushw y
#endif
    popw p
    ret
#ifdef ENABLE_isa_rloop
isa_rloop:
	mov r16,yl
	mov i0,r16
	hexb          ; show what byte
read1:
	rcall isa_in
	rjmp read1
#endif

#ifdef ENABLE_isa_tloop
isa_tloop:
	setport EN_CCMD,ENC_NODMA+ENC_PAGE1 ;       +ENC_STOP
	ldi r16,EN1_PHYS
	ldi r20,0x55
	ldi r17,6   ; ether size
lt1:
	mov i0,r20
	inc r20
	rcall isa_out
	inc r16 ; advance port
	dec r17
	brne lt1
	ret
#endif

#ifdef ENABLE_isa_wloop
isa_wloop:
	mov r16,yl
	mov i0,r20l
write1:
	rcall isa_out
	rjmp write1
#endif

#ifdef ENABLE_sram_write
sram_write:
	st y,r20
	ret
#endif

neon_toggle:
	lds r17,neon
	com r17
	sts neon,r17
	rjmp showtoggle

log_toggle:
	lds r17,nelog
	com r17
	sts nelog,r17
showtoggle:
	mov i0,r17
	hexb
	ret

#ifdef ENABLE_dump_net_header
dump_net_header:
	movw r20,y      ; get address into r20
	rcall ne_getheader
	movwi x,rcv_hdr
	movwi r16,hdr_size
	rcall hexmem_x
	ret
#endif

#ifdef ENABLE_board_write
board_write:
	movw r20,y      ; get address into r20
	movwi r18,hdr_size  ; write as much as a header is
	clrw y            ; we'll write y
	movwi z,wb1         ; data provider
	rcall ne_wstart
	ret
wb1:
	incw y
	mov i0,yl
	mov i1,i0
	com i1
	rjmp hexy
#endif

#ifdef ENABLE_board_dump
board_dump:
	movwi r20,0x4600        ; start at first receive buffer
	movwi r18,(SM_RSTOP_PG<<8)-(SM_RSTART_PG<<8)
	movwi z,bdd1
	rcall ne_rstart         ; start transfer
	ret

bdd1:
	tstbi r18l,0x1f         ; check if time for new line
	brne bdd2
	crlf
	movw y,r20
	rcall hexy
	space
bdd2:
	hexb
	mov i0,i1
	hexb             ; byte mode
	addwi r20,1
	ldi w,0x1f
	and w,r20l
	cpi w,0x10
	brne bdd3
	space
bdd3:
	ret
#endif

#ifdef ENABLE_ea_reg_diag
;;
;; ethernet register disgnostic
;;
ea_reg_diag:
	clr r17
	sts     nch,r17
	sts     ready,r17
	setport en_ccmd,enc_nodma+enc_page1

ea_reg_diag_1:

	ldi r17,6
	ldi r16,1               ; starting at port 1
next_ea_1:
	mov i0,r17
	rcall    isa_out
	inc r16
	dec r17
	brne next_ea_1

	ldi r17,6
	ldi r16,1               ; starting at port 1
next_ea_2:
	rcall    isa_in
	space
	hexb
	inc r16
	dec r17
	brne next_ea_2
	crlf

	lds     w,ready
	tst     w
	breq    ea_reg_diag_1            ; if CR not yet pressed...

	rcall iosweep

	setport en_ccmd,enc_nodma+enc_page0
	ret
#endif

#ifdef ENABLE_iosweep
iosweep:
	setport en_ccmd,enc_nodma+enc_page0
	rcall iosweep1
	setport en_ccmd,enc_nodma+enc_page1
	rcall iosweep1
	setport en_ccmd,enc_nodma+enc_page2
	rcall iosweep1
	setport en_ccmd,enc_nodma+enc_page3
	rcall iosweep1
	setport en_ccmd,enc_nodma+enc_page0
	ret
iosweep1:
;;
;; sweep the possible I/O space - see what's out there!
;;
	ldi r17,16              ; was 32 - but don't do it now for NE2000
	                        ; because upper ports if read without conditioning for
	                        ; transfer cause problems on REAL ISA bus - why???
	clr r16                 ; starting at port 0
nextboard:
	rcall    isa_in
	space
	hexb
	inc r16
	dec r17
	brne nextboard
	crlf
	ret
#endif

;
;   p-code command checker
;
;   r10 = 1- or 2-byte command code
;   r12 = routine to call if command codes match
;
pchk_cmd:  pcode_routine 2
	movw r18,r10            ; move target command bytes to r18
	movwi x,inbuf           ; move input buffer pointer to x
	ld r16,x+               ; get first character of input command buffer
	cp r16,r18              ; match with command?
	brne chk_cmd_exit       ; no...exit
	cpi r19,0               ; yes...more command bytes?
	breq chk_cmd_1          ; no...execute command
	ld r16,x+               ; get second character of input command buffer
	cp r16,r19              ; match with command?
	brne chk_cmd_exit       ; no...exit
chk_cmd_1:
	sts cmd_ok,r18          ; show good command
	incw x                  ; skip character
	rcall hexcvtw           ; get next hex word from command buffer
	sts cmdop1,i0           ; save as LSB of address
	sts cmdop1+1,i1         ; save as MSB of address

	incw x                  ; skip character
	rcall hexcvtw           ; get next hex word from command buffer
	mov r20,i0              ; r20/21 = parameter #2
	mov r21,i1              ;
	sts cmdop2,i0           ; save as LSB of count/word
	sts cmdop2+1,i1         ; save as MSB of count/word

	ldsw y,cmdop1           ; y = parameter #1
	movw z,r12              ; call routine
        pushw p
	icall
        popw p
#if defined(PCODE_DEBUG)
    popw y
    putchar 'R'
    rcall hexy
    pushw y
#endif
chk_cmd_exit:
	ret

#ifdef ENABLE_zap_sram
zap_sram:               ; y has first parameter
	movwi x,0x60    ; base of SRAM
	movwi r18,512   ; size
	mov r20,yl      ; we'll write this value everywhere
zap_sram1:
	st x+,r20
	decw r18
	brne zap_sram1
	rjmp RESET      ; reset processor
#endif


show_ether:                     ; display ethernet address
    putchar 'e'
    putchar 'a'
    putchar ' '
    putchar '='
    movwi   x,my_ether
    ldi r16,6
    rjmp hexmem_x
