;-------------------------------------------------------------------------
; Copyright (c) 1999 Steven Freyder, David Helland, Bruce D. Lightner and
; Lightner Engineering.  Use subject to licensing restrictions described
; the file LICENSE.txt.  If you do not agree with the licensing terms then
; you must return and/or destroy this software immediately!
;-------------------------------------------------------------------------

;+
; **-seeprom_loader-net loader for seeprom.
;
; inputs:
;   @data_addr    = SEEPROM address
;   @data_addr+2... = data to load
;   @data_len     = tcp data byte count
;   buf           = network order destination port
;                   bit 0 =0 means EEPROM, =1 means SEEPROM
;
; outputs:
;   closes connection with "\n"
;-

#define SEEL_BATCH_SIZE ebuf_len
#define SEEL_BATCH_BUF  ebuf
#define SEEL_DEST   buf+14
#define SEEL_BLEN   buf+12
#define SEEL_ERRORS buf+10
#define SEEL_TEMP   buf+8
#define SEEL_LEN    buf+6
#define SEEL_EEADDR buf+4
#define SEEL_BOARD  buf+2
#define SEEL_WAIT   buf

seeprom_loader:
    pmovwi SEEL_DEST,[swap buf]                 ; save destination flag
    pr2s SEEL_EEADDR,[data_addr],2              ; get address
    pmovwi SEEL_BOARD,[data_addr]               ; board address for loop
    paddwi SEEL_BOARD,2                         ; +2 to skip address
    pmovwi SEEL_LEN,[data_len]                  ; get total data length
    psubwi SEEL_LEN,2                           ; minus address (2) = load len
    pjumpeq seeprom_loader_dump

    ;;;pmovbi putc_b,0	;!!!!!!!!!
    ;;;pprintv 0,[SEEL_EEADDR],0	;!!!!!!!!!
    ;;;pspace
    ;;;pprintv 0,[SEEL_LEN]	;!!!!!!!!!
    ;;;pcrlf	;!!!!!!!!!
    ;;;pmovbi putc_b,1	;!!!!!!!!1

    ;
    ; note well, the output CANNOT go to the board because ebuf is the
    ; same as boardbuf and using pputc during the load loop would overwrite
    ; boardbuf/ebuf!  output CAN be done once the load is done though.
    ;
seeprom_load_next_batch:
    pmovwi SEEL_BLEN,[SEEL_LEN]                 ; get remaining count
    pcmpwi SEEL_BLEN,SEEL_BATCH_SIZE            ; vs. batch size
    pjumplo seeprom_last_batch                  ; last one
    pmovwi SEEL_BLEN,SEEL_BATCH_SIZE            ; truncate to batch size
seeprom_last_batch:
    pr2s SEEL_BATCH_BUF,[SEEL_BOARD],[SEEL_BLEN]    ; read next batch
    pbitwi SEEL_DEST,1                          ; check destination
    pjumpne seeprom_load_seeprom                ; loading serial EEPROM
    ps2ee [SEEL_EEADDR],SEEL_BATCH_BUF,[SEEL_BLEN]
    pjump seeprom_load_wait_done

seeprom_load_seeprom:
    ps2see [SEEL_EEADDR],SEEL_BATCH_BUF,[SEEL_BLEN]
    pclrw SEEL_WAIT
seeprom_load_wait:
    pseeprom_poll SEEL_TEMP                     ; 0=non-busy
    paddwi SEEL_WAIT,1
    psubwi SEEL_TEMP,0
    pjumpne seeprom_load_wait

seeprom_load_wait_done:
    paddwi SEEL_EEADDR,SEEL_BATCH_SIZE
    paddwi SEEL_BOARD,SEEL_BATCH_SIZE
    psubwi SEEL_LEN,[SEEL_BLEN]             ; minus size of last batch
    pjumpne seeprom_load_next_batch
    pprintv 0,[SEEL_WAIT]
    pcrlf
    pret
;
; this code dumps 32 bytes at the requested address into the net buffer.
;
seeprom_loader_dump:
    pmovwi SEEL_LEN,32
seeprom_loader_dump_next:
    pbitwi SEEL_DEST,1
    pjumpne seeprom_dump_seeprom
    pee2s SEEL_WAIT,[SEEL_EEADDR],1
    pjump seeprom_dump_advance
seeprom_dump_seeprom:
    psee2s SEEL_WAIT,[SEEL_EEADDR],1
seeprom_dump_advance:
    pputc [SEEL_WAIT]                   ; put byte to 'net
    paddwi SEEL_EEADDR,1
    psubwi SEEL_LEN,1
    pjumpne seeprom_loader_dump_next
    pret
