;-------------------------------------------------------------------------
; Copyright (c) 1999 Steven Freyder, David Helland, Bruce D. Lightner and
; Lightner Engineering.  Use subject to licensing restrictions described
; the file LICENSE.txt.  If you do not agree with the licensing terms then
; you must return and/or destroy this software immediately!
;-------------------------------------------------------------------------

#define BUF_WRAP    (max_nch-1)
.dseg

serial_mode:        .byte   1   ; serial port mode 0=command, non-zero=binary
com_in: .byte   1
com_out:    .byte   1           ; com buffer in/out indices
com_lost:   .byte   1           ; count of lost bytes
xoff:       .byte   1           ; zero if not xoffed
ccaddr:     .byte   2           ; address where Control-C pressed
.cseg

;+
; **-serial_init-serial port initialization for reset.
;-

serial_init:
    clr r0
    sts xoff,r0                 ; not xoffed
    sts putc_b,r0               ; putchar is "normal"
    inc r0
    sts putcok,r0               ; putchar is enabled

; set serial port baud rate.

    ldi     r17,SERIAL_BAUD_DIVISOR
    out     UBRR,r17

; enable transmitter and receiver, and enable receiver interrupts.

    clr r10l
    rjmp pser_mode_init

com_circular:
    lds r17,com_in      ; get IN index
    lds r18,com_out     ;  and OUT index
    inc r17
    andi r17,BUF_WRAP   ; wrap it
    cp r17,r18          ; check it
    breq count_lost     ; sorry - we lost a char

    pushw z
    clr zh
    lds zl,com_in
    addwi z,inbuf
    in r18,udr
    st z,r18
    popw z

    sts com_in,r17      ; advance IN index
    rjmp com_bail       ; bail out
count_lost:
    in r18,udr          ; dump the byte to clear interrupt
    lds r17,com_lost
    inc r17
    sts com_lost,r17
    rjmp com_bail

rx_isr:
    push w
    push    r17
    push    r18
    in      r18,sreg
    push r18
    lds r18,serial_mode
    tst r18
    brne com_circular
    in      r17,udr
    cpi     r17,3       ; Control-C does a reset
    breq    gores
    push    yl
    push    yh
    cpi r17,0x13    ; control-S - set xof flag
    breq go_xoff
    cpi r17,0x11    ; control-Q - clear xof flag
    breq go_xoff
    lds     r18,ready
    tst     r18
    brne    rxignore
    cpi     r17,0x0d
    brne    nocr
is_cr:
    sts     ready,r17
    rjmp    rxignore
go_xoff:
    andi    r17,2       ; isolate on/off bit
    sts xoff,r17
    rjmp rxignore
nocr:
    cpi r17,0x0a        ; also allow LF
    breq is_cr
    lds     yl,nch
    ldi yh,0
    cpi     yl,max_nch
    brlt    rxok
    ldi     r17,'!'
    rjmp    rxecho
rxok:
    mov     r18,yl
    addwi y,inbuf
    st      y,r17
    inc     r18
    sts     nch,r18
rxecho:
    out     udr,r17
rxignore:
    pop     yh
    pop     yl
com_bail:
    pop     r18
    out     sreg,r18
    pop     r18
    pop     r17
    pop w
    reti
gores:
    pop r17
    pop r17
    pop r17
    pop r17

; stack is in same state as on initial entry to interrupt.

    pop yh
    pop yl
    stsw ccaddr,y
    clr r0
    out     UCR,r0          ; disable everything
    rjmp control_c
;+
; **-pser_mode-flush buffers and set mode on serial port.
;
; inputs:
;   r10l = mode as: 0-normal, non-zero-enable pass-all (binary) mode.
;
; outputs:
;   buffer flushed and characters being stored appropriately.
;-

pser_mode:  pcode_routine   1
pser_mode_init:
    ldi r17,(1<<rxen)|(1<<txen)                ; disable receive interrupts
    out UCR,r17

    clr r10h
    sts com_in,r10h	                       ; reset buffer pointers
    sts com_out,r10h
    sts nch,r10h                               ; reset command buffer

    tst r10l                                   ; set serial_mode using r10l
    breq psb_normal
    inc r10h
psb_normal:
    sts serial_mode,r10h

    ldi r17,(1<<rxcie)|(1<<rxen)|(1<<txen)     ; enable receive interrupts
    out UCR,r17
    ret
;+
; **-pser_putc-pcode serial port putchar.
;
; inputs:
;   r10 = character to transmit
;
; outputs:
;   transmission to serial port initiated.
;
; notes:
;   waits for a "transmit done" indication on the serial port
;   before transmitting this character.
;-

pser_putc:  pcode_routine 1
    mov ch,r10l
    rjmp do_putchar
;+
; **-pser_getc-pcode serial port getchar.
;
; outputs:
;   r10 -> resulting byte
;   condition codes set so that a jumpeq will take if buffer empty.
;-

pser_getc:  pcode_routine   1
    lds r17,com_in
    lds r18,com_out
    cp r17,r18
    brne pser_getc_got_byte
    rjmp set_pcode_flags
pser_getc_got_byte:
    movw x,r10              ; where to store it
    mov zl,r18              ; get OUT index low 1/2
    clr zh                  ; clear top 1/2
    addwi z,inbuf           ; bias for buffer
    ld r12l,z               ; get the byte
    st x,r12l               ; store it
    inc r18
    andi r18,BUF_WRAP       ; wrap it
    sts com_out,r18         ; store it back
    clz
    rjmp set_pcode_flags    ; jumpne will take now
;+
; **-serial_putchar-serial port putchar.
;
; inputs:
;   ch  = character
;   putcok  = 0 if putchar is disabled
;   xoff    = 0 if not xof'ed
;
; outputs:
;   character transmission started if appropriate
;
; notes:
;   this routine is called from the base PICOWEB module.
;-

import_serial_putchar:
serial_putchar:
    push r17
    lds r17,putcok
    tst r17
    breq putc_off
wait_xon:
    lds r17,xoff            ; check xof flag
    or r17,r17
    brne wait_xon
do_put1:
    in      r17,usr
    bst     r17,udre        ; check if data register empty
    brtc    do_put1
    out     udr,ch          ; start transmission
putc_off:
    pop r17
    ret
